#!/usr/bin/perl
# -*- project-name: VASM -*-

use strict;
use warnings;
use VASM::Resource;
use VASM::Resource::Catalog::Message;
use VASM::Resource::Catalog::Action;
use File::Spec::Functions qw/curdir rel2abs splitpath/;
use UI::Dialog::Backend::CDialog;
use File::MimeInfo::Magic;
use Archive::Tar;

# Message files
my $CommonMsg = findMessageCatalog(qw/Common Message/);
my $UICommonMsg = findMessageCatalog(qw/Action UI Common Message/);
my $UIMsg = findMessageCatalog(qw/Action UI CDialog Message/);
my $ErrorMsg = findErrorCatalog(qw/Action UI Error Message/);

# Dialog instance
my $d = UI::Dialog::Backend::CDialog->new(
  backtitle => $CommonMsg->Render('VASM Backtitle'),
  title => $UIMsg->Render('Title'),
  'ok-label' => $CommonMsg->Render('OK'),
  'cancel-label' => $CommonMsg->Render('Exit'),
  'yes-label' => $CommonMsg->Render('Yes'),
  'no-label' => $CommonMsg->Render('No'),
  width => 75,
  height => 20,
  menuheight => 10
);

sub mainMenu {
  while (1) {
    # Prompt choice
    my $mediaType = $d->menu(
      text => $UIMsg->Render('Main Menu Description'),
      list => [ map { $UICommonMsg->Render($_), 
                      $UIMsg->Render("$_ Description") }
                qw/File URI Template/ ]);

    # Act on input
    if ($d->state eq 'OK') {
      if ($mediaType eq 'File' or $mediaType eq 'URI') {
        mediaMenu($mediaType);
      } elsif ($mediaType eq 'Template') {
        selectTemplate();
      }
    } elsif ($d->state eq 'CANCEL') {
      exit 0;
    }
  }
}

sub mediaMenu {
  my ($mediaType) = @_;
  my $directory = mediaDirectory($mediaType);

  while (1) {
    ### Read
    my $member = $d->menu(
      text => $UIMsg->Render('Media Menu Description'),
      'extra-button' => 1,
      'extra-label' => $CommonMsg->Render('Back'),
      list => [ map { $_, $directory->Render($_) } sort $directory->List ]
    );
    my @path = ($mediaType, split(m!/!, $member));

    ### Evaluate
    if ($d->state eq 'OK') {
      operationMenu(@path);
    } elsif ($d->state eq 'EXTRA') {
      return;
    } elsif ($d->state eq 'CANCEL') {
      exit 0;
    }
  }
}

sub operationMenu {
  my @path = @_;
  my $mediaType = (@path == 3) ? join '/', @path[1..2] : $path[1];

  while (1) {
    ### Read
    my $operation = $d->menu(
      text => $UIMsg->Render(id => 'Operation Menu Description',
                             arguments => [ $mediaType ]),
      'extra-button' => 1,
      'extra-label' => $CommonMsg->Render('Back'),
      list => [ map { $UIMsg->Render("$_ Operation"),
                      $UIMsg->Render("$_ Operation Description") }
                qw/List Add Clear/ ]
    );

    ### Evaluate
    if ($d->state eq 'OK') {
      if ($operation eq 'List') {
        mediumList(@path);
      } elsif ($operation eq 'Add') {
        mediumAdd(@path);
      } elsif ($operation eq 'Clear') {
        mediumClear(@path);
      }
    } elsif ($d->state eq 'EXTRA') {
      return;
    } elsif ($d->state eq 'CANCEL') {
      exit 0;
    }
  }
}

sub mediumList {
  my @path = @_;
  my $mediaType = (@path == 3) ? join '/', @path[1..2] : $path[1];
  
  # Try to find a matching actions catalog
  my $catalog = findActionCatalog(@path);

  if (defined $catalog) {
    $d->msgbox(text => $UIMsg->Render(id => 'Commands Listed',
                                      arguments => [ $mediaType ])
                      . ":\\n\n\\n\n" # Yes, I know this this is ugly
                      . join("\\n\n", $catalog->Dump));
  } else { # Could not find a catalog!
    $d->msgbox(text => $ErrorMsg->Render(id => 'No Action Catalog For Medium',
                                         arguments => [ $mediaType ]));
  }

  return;
}

sub mediumAdd {
  my @path = @_;

  # First, load the action catalog, if possible
  my $catalog = findActionCatalog(@path);
  
  # If it didn't exist, that's OK; just instantiate a blank one
  $catalog = VASM::Catalog::Action->new unless defined $catalog;

  my $command = $d->inputbox(text => $UIMsg->Render('Add Command'),
                             'cancel-label' => $CommonMsg->Render('Back'));

  if ($d->state eq 'OK') {
    # Emit warning message and return unless the command is a valid value
    $d->msgbox(text => $ErrorMsg->Render('Null Command String')) and return
      unless length $command;

    $catalog->Unshift($command); # Unshift it onto the catalog instance
    writeActionCatalog($catalog, @path); # Now write the instance out
  } 
  
  return;
}

sub mediumClear {
  my @path = @_;
  my $mediaType = (@path == 3) ? join '/', @path[1..2] : $path[1];
  
  # Try to find a matching actions catalog
  my $catalog = findActionCatalog(@path);

  if (defined $catalog) {
    # Confirm this destructive operation
    my $confirm = $d->yesno(
      text => $UIMsg->Render(id => 'Confirm Clear',
                             arguments => [ $mediaType ])
    );
    if ($confirm) { # "Do it!" -- Yellowman
      $catalog->Clear;
      writeActionCatalog($catalog, @path);
    }
  } else { # Could not find a catalog!
    $d->msgbox(text => $ErrorMsg->Render(id => 'No Action Catalog',
                                         arguments => [ $mediaType ]));
  }
}

BEGIN {
  # selectTemplate has its own 'static' variable for the current working
  # directory; see perlfaq7
  my $wd = rel2abs(curdir);
  
  sub selectTemplate {
    # Get the filename of a template archive; start at current directory
    my $template = $d->fselect(path => $wd);
    
    if ($d->state eq 'OK') {
      # Confirm this destructive operation
      my $confirm = $d->yesno(text => $UIMsg->Render('Confirm Install'));
      
      if ($confirm) { # Do it...
        # Store new current working directory
        $wd = (splitpath($wd))[1];
          
        # Check the MIME type of the file
        my $mimetype = mimetype($template);
          
        $d->msgbox(text => $ErrorMsg->Carp('Invalid Archive')) and return
          unless $mimetype =~ m!^application/x-(?:compressed-)?tar$!;

        # Find the resource path and create it if necessary
        my $root = makeConfigResourcePath('Action');

        # Change directory and extract the template archive
        chdir $root; Archive::Tar->extract_archive($template);

        # Confirm the installation (to the user)
        $d->msgbox(text => $UIMsg->Render('Installation Complete'));
      }
    }
    
    return;
  }
}

# Start the application
mainMenu();
